/* ************************************************************************** */
/*                                       _____           _                    */
/*  libp7/internals.h                   |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : libp7                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/05/12 01:16:06                               |___/     */
/*                                                                            */
/* ************************************************************************** */

#ifndef LIBP7_INTERNALS_H
# define LIBP7_INTERNALS_H
# include <libp7/streams.h>
# include <libp7/packetio.h>
# include <libp7.h>

/* Buffer sizes */
/**
 *	MAX_RAWDFLD_SIZE:
 *	Max raw data field size.
 *
 *	> id (4 bytes) + total (4 bytes) + data
 */

# define MAX_RAWDFLD_SIZE (8 + MAX_RAWDATA_SIZE)

/**
 *	MAX_ENCDFLD_SIZE:
 *	Max encoded data field size.
 *
 *	> id (4 bytes) + total (4 bytes) + encoded data
 *	The worst case is where every byte has to be encoded (an '\' has to be
 *	added), so the size of the worst (biggest) case is MAX_RAWDFLD_SIZE * 2.
 */

# define MAX_ENCDFLD_SIZE (8 + (MAX_RAWDATA_SIZE * 2))

/**
 *	MAX_PACKET_SIZE:
 *	Max packet size.
 *
 *	Used to be :
 *	> t + st + ext + encoded D field + checksum + nullbyte
 *	> 8 + MAX_ENCDFLD_SIZE + 2 + 1
 *
 *	But with OHP (screen streaming) introduction, this size wasn't enough to
 *	store screen streaming packets. So we use the OHP packet size as a
 *	reference, even for normal protocol, in case user launches "normal" packet
 *	mode and the calculator sends OHP packets. The formula is :
 *	> ohp type + pic data + checksum
 */

# define MAX_PACKET_SIZE (6 + 1024 + 2)

/* ************************************************************************** */
/*  Packet I/O                                                                */
/* ************************************************************************** */
/* Utilities */
int p7_send_basic(unsigned int type, unsigned int subtype, int resp);
int p7_send_ext(unsigned int type, unsigned int subtype,
	void *data, unsigned int size, int resp);
int p7_send_again(void);
int p7_recv(int checksum, int wasresend);

/* Resend error packet (meant to be used with recv, not suited for public) */
int p7_send_err_resend(void);

/* ************************************************************************** */
/*  Useful globals                                                            */
/* ************************************************************************** */
/* Stream */
extern FILE *p7_stream;

/* Is active */
extern int p7_active;

/* Is shifted */
extern int p7_shifted;

/* last sent command */
extern unsigned int p7_last_sent_command;

/* ************************************************************************** */
/*  Buffers                                                                   */
/* ************************************************************************** */
/* raw sending packet buffers */
extern unsigned char *p7_send_buffers[2];
extern int p7_send_buffers_size[2];
extern int p7_send_buffer_id;
/* raw reception packet buffer */
extern unsigned char *p7_recv_buffer;

/* active response */
# define response (*p7_response)

/* ************************************************************************** */
/*  Utilities                                                                 */
/* ************************************************************************** */
/* Encoding/decoding functions */
unsigned int p7_encode(void *fnal, void *raw, unsigned int size);
unsigned int p7_decode(void *fnal, void *encoded, unsigned int size);

/* Checksum function */
unsigned int p7_checksum(unsigned char *packet, unsigned int size);

/* ASCII-hex utilities */
void p7_putascii(unsigned char *p, unsigned int i, int n);
unsigned int p7_getascii(const unsigned char *p, int n);

# include <libp7/internals/log.h>
#endif
