/* ************************************************************************** */
/*                                       _____           _                    */
/*  geterror.c                          |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : libp7                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/08/17 00:49:07                               |___/     */
/*                                                                            */
/* ************************************************************************** */

#include <libp7/internals.h>

/**
 *	p7_geterror:
 *	Get a string description of the error.
 *
 *	This function MUST evolve whenever the p7_error_t enumeration
 *	is updated. Could cause invalid error strings otherwise,
 *	or segfaults !
 *
 *	@return				error string description
 */

char *p7_geterror(void)
{
	/* errors */
	static char *errors[] = {
		"no error have been encountered",

		/* errors caused by libusb or libusb usage */
		"device is uninitialized",
		"libusb context could not be created",
		"couldn't get calculator (is it plugged and in receive mode ?)", 
		"couldn't get access to calculator",

		/* misc transmission things */
		"wrong mode (was active when passive is required, or other way round)",
		"calculator sent unknown type (OHP/Receive mode confusion ?)",
		"transmission timed out",
		"checksum was invalid", /* not actually meant to be used */
		"irrecoverable (bad luck), try again",

		/* user interface */
		"given file stream is unseekable",
		"filestream musn't be empty !",
		"filename is missing or too long (12 chars max)",
		"directory name is too long (8 chars max)",
		"overwrite impossible (invalid device/requested file doesn't exist)",
		"not enough space left",
		"we denied overwrite",

		/* others */
		"unsupported command (comes from Receive mode/OS Update confusion)",
		"an internal error has occurred (see logs)",
		"an unknown error has occurred",
	};

	/* send error corresponding to global code */
	return (errors[p7_error]);
}
