/* ************************************************************************** */
/*                                       _____           _                    */
/*  init.c                              |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : libp7                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/08/16 23:19:46                               |___/     */
/*                                                                            */
/* ************************************************************************** */

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <libp7/internals.h>
#define casio_vendor 0x07CF
#define casio_product 0x6101

/* ************************************************************************** */
/*  Global vars                                                               */
/* ************************************************************************** */
/**
 *	p7_error:
 *	The error.
 */

int p7_error = p7_noerror;

/**
 *	p7_active:
 *	Boolean to know if we are in active or passive mode.
 */

int p7_active;

/**
 *	p7_shifted:
 *	Boolean to know if we are shifted.
 */

int p7_shifted;

/**
 *	p7_stream:
 *	Main reading/writing stream.
 */

FILE *p7_stream = NULL;

/* ************************************************************************** */
/*  Main functions                                                            */
/* ************************************************************************** */
/**
 *	p7_exit:
 *	De-initialize libp7.
 */

static void p7_exit(void)
{
	/* free buffers */
	/* - raw reception packet buffer - */
	if (p7_recv_buffer) free(p7_recv_buffer);
	/* - raw sending packet buffers - */
	if (p7_send_buffers[0]) free(p7_send_buffers[0]);
	if (p7_send_buffers[1]) free(p7_send_buffers[1]);
	/* - decoded packet buffer - */
	if (p7_response) free(p7_response);

	/* close stream */
	if (p7_stream) fclose(p7_stream);
	p7_stream = NULL;
}

/**
 *	p7_init_stream:
 *	Initialize libp7 with stream.
 *
 *	@arg	stream		the filestream
 *	@arg	active		start as active ?
 *	@return				if it worked (0 if no)
 */

int p7_init_stream(FILE *stream, int active)
{
	/* set libp7 exit function */
	atexit(&p7_exit);

	/* allocating buffers */
	if (!(p7_send_buffers[0] = malloc(MAX_PACKET_SIZE)))
		return (0);
	if (!(p7_send_buffers[1] = malloc(MAX_PACKET_SIZE)))
		return (0);
	if (!(p7_recv_buffer = malloc(MAX_PACKET_SIZE + 1)))
		return (0);
	if (!(p7_response = malloc(sizeof(p7_packet_t))))
		return (0);

	/* starting as active */
	p7_active = active;

	/* set stream */
	p7_stream = stream;

	/* initialization went alright */
	return (1);
}
