/* ************************************************************************** */
/*                                       _____           _                    */
/*  protocol/getscreen.c                |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : libp7                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/09/06 16:48:03                               |___/     */
/*                                                                            */
/* ************************************************************************** */

#include <libp7/internals.h>

/**
 *	p7_getscreen:
 *	Get the screen.
 *
 *	@arg	callback	the main callback for the function.
 *						for each correctly received frame, the function will
 *						be called with the VRAM as a parameter.
 *						if it returns 0, the function stops.
 *	@return				if it worked
 */

int p7_getscreen(int (*callback)(unsigned char*))
{
	/* check mode */
	if (p7_active) {
		p7_error = p7_error_active;
		log_error("expected passive !");
		return (0);
	}

	/* main loop */
	while (1) {
		/* get packet */
		if (!p7_recv(0, 0))
			return (0);

		/* check type */
		if (response.type != p7_pt_ohp) {
			p7_error = p7_error_unknown;
			return (0);
		}

		/* then call back the callback */
		if (!(*callback)(response.screen.vram))
			break;
	}

	/* stop */
	return (1);
}
